package rs.fon.is.weka.classification;

import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import rs.fon.is.weka.domain.WeatherData;
import rs.fon.is.weka.util.RWXMLWeatherData;
import weka.classifiers.Evaluation;
import weka.classifiers.bayes.NaiveBayes;
import weka.classifiers.evaluation.NominalPrediction;
import weka.classifiers.evaluation.Prediction;
import weka.core.Attribute;
import weka.core.DenseInstance;
import weka.core.Instance;
import weka.core.Instances;


public class NaiveBayesClassificatorInput {
	
	private static final String trainingDataSource = "data/weatherTrainData.xml";
	private static final String testDataSource = "data/weatherTestData.xml";
	
	private static DecimalFormat formatter = new DecimalFormat();
	
	public static void main(String[] args) throws Exception {
		
		// Create attributes: outlook, temperature, humidity, windy, play
		ArrayList<Attribute> attributes = createAttributes();
			
		// Create training dataset (by reading data from the xml data file)
		List<WeatherData> weatherData = RWXMLWeatherData.readWeatherDataFromFile(trainingDataSource);
		Instances trainDataSet = createDataSet("ToPlayOrNotToPlayTrain", attributes, weatherData);
		
		// Create the Naive Bayes Classifier
		NaiveBayes nbClsf = new NaiveBayes();
		nbClsf.buildClassifier(trainDataSet);
		
		System.out.println(nbClsf);
		
		// Create test dataset (by reading data from the xml data file)
		List<WeatherData> testDataSrc = RWXMLWeatherData.readWeatherDataFromFile(testDataSource);
		Instances testDataSet = createDataSet("ToPlayOrNotTopalyTest", attributes, testDataSrc);
		
		// Evaluate the model with the test dataset
		Evaluation eval = new Evaluation(testDataSet);
		eval.evaluateModel(nbClsf, testDataSet);
		
		// Print the confusion matrix
		System.out.println(eval.toMatrixString());
		
		// Print the predictions over the test dataset
		System.out.println("Inst.\tPred.\tActual\tError\tProb.Distribution");
		System.out.println("----------------------------------------------");
		
		int k = 1;
		ArrayList<Prediction> predictions = eval.predictions();
		for (Prediction prediction : predictions) {
			NominalPrediction nomPred = (NominalPrediction)prediction;
			
			int actual = (int)nomPred.actual();
			int predicted = (int)nomPred.predicted();
			double[] distribution = nomPred.distribution();
			
			System.out.print(k++);
			System.out.print("\t" + attributes.get(4).value(predicted));
			System.out.print("\t" + attributes.get(4).value(actual));
			System.out.print("\t" + (actual == predicted ? " " : "+"));
			System.out.println("\t" + formatter.format(distribution[0]) + "\t" + formatter.format(distribution[1]));
		}
		
	}
	
	private static ArrayList<Attribute> createAttributes() {
		ArrayList<Attribute> attributes = new ArrayList<>();
		
		//outlook: sunny, overcast, rainy
		ArrayList<String> outlookVals = new ArrayList<>();
		outlookVals.add("sunny");
		outlookVals.add("overcast");
		outlookVals.add("rainy");
		attributes.add(new Attribute("outlook", outlookVals));
		
		//temperature: hot, mild, cool 
		ArrayList<String> tempVals = new ArrayList<>();
		tempVals.add("hot");
		tempVals.add("mild");
		tempVals.add("cool");
		attributes.add(new Attribute("temperature", tempVals));
			
		//humidity: high, normal
		ArrayList<String> humidityValues = new ArrayList<String>();
		humidityValues.add("high");
		humidityValues.add("normal");
		attributes.add(new Attribute("humidity", humidityValues));
		
		//windy: true, false
		ArrayList<String> windyValues = new ArrayList<String>();
		windyValues.add("false");
		windyValues.add("true");
		attributes.add(new Attribute("windy", windyValues));
		
		//play: yes, no
		ArrayList<String> playNominalValues = new ArrayList<String>();
		playNominalValues.add("yes");
		playNominalValues.add("no");
		attributes.add(new Attribute("play", playNominalValues));
		
		return attributes;
	}
	
	private static Instances createDataSet(String dataSetName, List<Attribute> attributes, List<WeatherData> weatherData) {
		//create empty dataset
		Instances dataSet = new Instances(dataSetName, (ArrayList<Attribute>) attributes, weatherData.size());
		//add instances
		for (WeatherData wData : weatherData) {
			dataSet.add( createInstance(attributes, wData));
		}	
		// set the index of the class attribute
		dataSet.setClassIndex(dataSet.numAttributes()-1);
		
		return dataSet;
	}

	private static Instance createInstance(List<Attribute> attributes, WeatherData dataItem) {
		
		Instance i = new DenseInstance(attributes.size());
		i.setValue(attributes.get(0), dataItem.getOutlook());
		i.setValue(attributes.get(1), dataItem.getTemperature());
		i.setValue(attributes.get(2), dataItem.getHumidity());
		i.setValue(attributes.get(3), dataItem.getWindy());
		i.setValue(attributes.get(4), dataItem.getPlay());
		
		return i;
	}
}
